use crate::Cfg;
use crate::Target;
use std::fmt::{Display, Formatter, Result};

impl Cfg {
    fn fmt_list(list: &[Cfg], f: &mut Formatter<'_>) -> Result {
        if !list.is_empty() {
            let last = list.len() - 1;
            for (i, e) in list.iter().enumerate() {
                e.fmt_bare(f)?;
                if i != last {
                    f.write_str(", ")?;
                }
            }
        }
        Ok(())
    }

    fn fmt_bare(&self, f: &mut Formatter<'_>) -> Result {
        match self {
            Cfg::Is(k) => f.write_str(k),
            Cfg::Equal(k, v) => {
                f.write_str(k)?;
                f.write_str(" = \"")?;
                f.write_str(v)?; // FIXME: once unescaping is fixed, this one will need too
                f.write_str("\"")
            },
            Cfg::Any(e) => {
                f.write_str("any(")?;
                Self::fmt_list(e, f)?;
                f.write_str(")")
            },
            Cfg::All(e) => {
                f.write_str("all(")?;
                Self::fmt_list(e, f)?;
                f.write_str(")")
            },
            Cfg::Not(e) => {
                f.write_str("not(")?;
                e.fmt_bare(f)?;
                f.write_str(")")
            },
        }
    }
}

impl Display for Cfg {
    fn fmt(&self, f: &mut Formatter<'_>) -> Result {
        f.write_str("cfg(")?;
        self.fmt_bare(f)?;
        f.write_str(")")
    }
}

impl Display for Target {
    fn fmt(&self, f: &mut Formatter<'_>) -> Result {
        match self {
            Target::Cfg(c) => c.fmt(f),
            Target::Triple {arch, vendor, os, env} => {
                f.write_str(arch)?;
                f.write_str("-")?;
                f.write_str(vendor)?;
                f.write_str("-")?;
                f.write_str(os)?;
                if let Some(env) = env {
                    f.write_str("-")?;
                    f.write_str(env)
                } else {
                    Ok(())
                }
            }
        }
    }
}

#[test]
fn print() {
    assert_eq!("cfg(ok)", "cfg( ok )".parse::<Target>().unwrap().to_string());
    assert_eq!("foo-bar-baz", "foo-bar-baz".parse::<Target>().unwrap().to_string());
    assert_eq!("foo-bar-baz-quz", " foo-bar-baz-quz ".parse::<Target>().unwrap().to_string());
    assert_eq!("cfg(foo)", &Cfg::Is("foo".to_string()).to_string());
    assert_eq!("cfg(not(foo))", &Cfg::Not(Box::new(Cfg::Is("foo".to_string()))).to_string());
    assert_eq!("cfg(not(any(foo, bar)))", &Cfg::Not(Box::new(Cfg::Any(vec![
        Cfg::Is("foo".to_string()),
        Cfg::Is("bar".to_string()),
    ]))).to_string());
}
